# This is a part of the external Repeat One Song plugin for Rhythmbox
#
# Author: Eduardo Mucelli Rezende Oliveira
# E-mail: edumucelli@gmail.com or eduardom@dcc.ufmg.br
# Version: 0.4 (Unstable) for Rhythmbox 3.0.1 or later
#
#
# reworked for alternative-toolbar
# Author: fossfreedom
#
# This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

# This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

from gi.repository import GLib
from gi.repository import GObject
from gi.repository import Gdk
from gi.repository import Gio
from gi.repository import Gtk

from alttoolbar_preferences import CoverLocale
from alttoolbar_preferences import GSetting
from alttoolbar_rb3compat import gtk_version


class Repeat(GObject.Object):
    def __init__(self, shell, toggle_button):
        """

        :param toggle_button:  button that controls the repeat functions
        :return:
        """
        GObject.Object.__init__(self)

        # use this to start the repeat-one-song capability (if True)
        self.repeat_song = False
        self.shell = shell
        self.toggle_button = toggle_button

        # self.one_song_stprint ("adjoining")
        ate_normal, self.one_song_state_eos = range(2)
        # self.one_song_state = self.one_song_state_normal

        player = self.shell.props.shell_player
        # Please refer to the comments above to understand why those
        # two callbacks are not being used currently, Rhytmbox 2.99.1
        # player.connect('playing-song-changed', self.on_song_change)
        # player.props.player.connect('eos', self.on_gst_player_eos)
        player.connect('elapsed-changed', self.on_elapsed_change)

        if gtk_version() >= 3.12:
            popover = Gtk.Popover.new(toggle_button)

            repeat = RepeatPopContainer(popover, toggle_button)
            popover.add(repeat)
            popover.set_modal(False)

        else:
            # use our custom Popover equivalent for Gtk+3.10 folks
            popover = CustomPopover(toggle_button)
            repeat = RepeatPopContainer(popover, toggle_button)
            popover.add(repeat)

        toggle_button.connect("toggled", self._on_toggle, popover, repeat)
        repeat.connect('repeat-type-changed', self._on_repeat_type_changed)

        self._on_repeat_type_changed(repeat, repeat.get_repeat_type())

    def _on_toggle(self, toggle, popover, repeat):
        if toggle.get_active():
            popover.show_all()
            self.repeat_song = \
                repeat.get_repeat_type() == RepeatPopContainer.ONE_SONG
        else:
            popover.hide()
            self.repeat_song = False

        self._set_toggle_tooltip(repeat)

        print("on toggle", self.repeat_song)

    def _set_toggle_tooltip(self, repeat):
        # locale stuff
        cl = CoverLocale()
        cl.switch_locale(cl.Locale.LOCALE_DOMAIN)
        if self.toggle_button.get_has_tooltip():
            if repeat.get_repeat_type() == RepeatPopContainer.ALL_SONGS:
                message = _("Repeat all tracks")
            else:
                message = _("Repeat the current track")
            self.toggle_button.set_tooltip_text(message)
        cl = CoverLocale()
        cl.switch_locale(cl.Locale.RB)

    def _on_repeat_type_changed(self, repeat, repeat_type):
        if self.toggle_button.get_active():
            if repeat_type == RepeatPopContainer.ONE_SONG:
                self.repeat_song = True
            else:
                self.repeat_song = False
        else:
            self.repeat_song = False

        self._set_toggle_tooltip(repeat)

        print("repeat type changed", self.repeat_song)

    # Looks like there is a bug on gstreamer player and a seg fault
    # happens as soon as the 'eos' callback is called.
    # https://bugs.launchpad.net/ubuntu/+source/rhythmbox/+bug/1239218
    # As soon it gets fixed or a code-based workaround gets available,
    # this method in conjunction with on_song_change will be used as
    # the way to control the song repetition. Meanwhile, on_elapsed_change
    # will be the chosen solution
    def on_gst_player_eos(self, gst_player, stream_data, early=0):
        # EOS signal means that the song changed because the song is over.
        # ie. the user did not explicitly change the song.
        # https://developer.gnome.org/rhythmbox/
        #                         unstable/RBPlayer.html#RBPlayer-eos
        if self.repeat_song:
            self.one_song_state = self.one_song_state_eos

    # This is a old method to 'repeat' the current song as soon as it
    # reaches the last second. Will be the used until the bug mentioned on the
    # comments above gets fixed.
    def on_song_change(self, player, time):
        if self.one_song_state == self.one_song_state_eos:
            self.one_song_state = self.one_song_state_normal
            player.do_previous()

    # This is a old method to 'repeat' the current song as soon as it
    # reaches the last second. Will be the used until the bug mentioned on the
    # comments above gets fixed.
    # This might be improved keeping a instance variable with the duration and
    # updating it on_song_change. Therefore, it would not be
    # necessary to query the duration every time
    def on_elapsed_change(self, player, time):
        if self.repeat_song:
            duration = player.get_playing_song_duration()
            if duration > 0:
                # Repeat on the last two seconds of the song. Previously the
                # last second was used but RB now seems to use the last second
                # to prepare things for the next song of the list
                if time >= duration - 2:
                    player.set_playing_time(0)


class RepeatPopContainer(Gtk.ButtonBox):
    __gsignals__ = {
        "repeat-type-changed": (GObject.SIGNAL_RUN_LAST, None, (int,))
    }

    # repeat-type-changed is emitted with one of the following values
    ONE_SONG = 1
    ALL_SONGS = 2

    def __init__(self, parent_container, parent_button, *args, **kwargs):
        super(RepeatPopContainer, self).__init__(*args, **kwargs)

        self.set_orientation(Gtk.Orientation.HORIZONTAL)
        self.set_layout(Gtk.ButtonBoxStyle.START)
        self.props.margin = 5
        context = self.get_style_context()
        context.add_class('linked')

        icon_size = 4

        toggle1 = Gtk.RadioButton.new(None)
        toggle1.set_mode(False)
        fallback = 'media-playlist-repeat-symbolic'
        icon = Gio.ThemedIcon.new_with_default_fallbacks(fallback)
        image = Gtk.Image()
        image.set_from_gicon(icon, icon_size)
        image.props.margin = 5
        toggle1.set_image(image)
        toggle1.connect('leave-notify-event', self._on_popover_mouse_over)
        toggle1.connect('enter-notify-event', self._on_popover_mouse_over)
        toggle1.connect('toggled', self._on_popover_button_toggled)

        # locale stuff
        cl = CoverLocale()
        cl.switch_locale(cl.Locale.LOCALE_DOMAIN)

        if parent_button.get_has_tooltip():
            toggle1.set_tooltip_text(_("Repeat all tracks"))

        self._repeat_button = toggle1
        self.add(toggle1)
        self.child_set_property(toggle1, "non-homogeneous", True)
        toggle1.show_all()

        self._repeat_image = Gtk.Image()
        self._repeat_image.set_from_gicon(icon, icon_size)
        self._repeat_image.props.margin = 5

        toggle2 = Gtk.RadioButton.new_from_widget(toggle1)
        toggle2.set_mode(False)
        sym = 'media-playlist-repeat-song-symbolic'
        icon2 = Gio.ThemedIcon.new_with_default_fallbacks(sym)
        image2 = Gtk.Image()
        image2.set_from_gicon(icon2, icon_size)
        image2.props.margin = 5
        toggle2.set_image(image2)

        if parent_button.get_has_tooltip():
            toggle2.set_tooltip_text(_("Repeat the current track"))

        self._repeat_song_image = Gtk.Image()
        self._repeat_song_image.set_from_gicon(icon2, icon_size)
        self._repeat_song_image.props.margin = 5

        toggle2.connect('leave-notify-event', self._on_popover_mouse_over)
        toggle2.connect('enter-notify-event', self._on_popover_mouse_over)
        toggle2.connect('toggled', self._on_popover_button_toggled)
        toggle2.show_all()
        self._repeat_song_button = toggle2
        self.add(toggle2)
        self.child_set_property(toggle2, "non-homogeneous", True)

        self._popover_inprogress = 0
        parent_container.connect('leave-notify-event',
                                 self._on_popover_mouse_over)
        parent_container.connect('enter-notify-event',
                                 self._on_popover_mouse_over)
        parent_button.connect('leave-notify-event',
                              self._on_popover_mouse_over)
        parent_button.connect('enter-notify-event',
                              self._on_popover_mouse_over)

        parent_button.set_image(self._repeat_image)

        self._parent_container = parent_container
        self._parent_button = parent_button

        # now get the repeat-type saved in gsettings
        # get values from gsettings
        self.gs = GSetting()
        self.plugin_settings = self.gs.get_setting(self.gs.Path.PLUGIN)

        repeat_type = self.plugin_settings[self.gs.PluginKey.REPEAT_TYPE]

        if repeat_type == RepeatPopContainer.ONE_SONG:
            self._repeat_song_button.set_active(True)

    def _on_popover_button_toggled(self, button, *args):
        print("popover toggle")
        if button.get_active():
            if button == self._repeat_button:
                self._parent_button.set_image(self._repeat_image)
                self.emit('repeat-type-changed', RepeatPopContainer.ALL_SONGS)
                self.plugin_settings[self.gs.PluginKey.REPEAT_TYPE] = \
                    RepeatPopContainer.ALL_SONGS
            else:
                self._parent_button.set_image(self._repeat_song_image)
                self.emit('repeat-type-changed', RepeatPopContainer.ONE_SONG)
                self.plugin_settings[self.gs.PluginKey.REPEAT_TYPE] = \
                    RepeatPopContainer.ONE_SONG

    def get_repeat_type(self):
        repeat_type = RepeatPopContainer.ALL_SONGS
        if self._repeat_song_button.get_active():
            repeat_type = RepeatPopContainer.ONE_SONG

        return repeat_type

    def _on_popover_mouse_over(self, widget, eventcrossing):
        if eventcrossing.type == Gdk.EventType.ENTER_NOTIFY:
            if self._popover_inprogress == 0:
                self._popover_inprogress = 1
                print("enter1")
            else:
                self._popover_inprogress = 2
                print("enter2")
            self._popover_inprogress_count = 0

            if type(widget) is Gtk.ToggleButton:
                print("here")
                if widget.get_active():
                    print(self._parent_container)
                    self._parent_container.show_all()
        else:
            print("exit")
            self._popover_inprogress = 3

        def delayed(*args):
            if self._popover_inprogress == 3:
                self._popover_inprogress_count += 1

                if self._popover_inprogress_count < 5:
                    return True

                self._parent_container.hide()
                self._popover_inprogress = 0
                print("exit timeout")
                return False
            else:
                return True

        if self._popover_inprogress == 1:
            print("adding timeout")
            self._popover_inprogress = 2
            GLib.timeout_add(100, delayed)


class CustomPopover(Gtk.Window):
    def __init__(self, parent_button, *args, **kwargs):
        super(CustomPopover, self).__init__(type=Gtk.WindowType.POPUP, *args,
                                            **kwargs)

        self.set_decorated(False)
        self.set_resizable(False)
        self.set_type_hint(Gdk.WindowTypeHint.DOCK)
        self.stick()
        self._parent_button = parent_button
        self.connect_after('show', self._on_show)
        # Track movements of the window to move calendar window as well
        self.connect("configure-event", self.on_window_config)

    def add(self, widget):
        self._frame = Gtk.Frame()
        self._frame.add(widget)

        super(CustomPopover, self).add(self._frame)
        self._frame.show_all()

    # Popoverwindow co ordinates without off-screen correction:
    #         Window origin (x, y)
    #          |
    #          V
    #          ---------------------------------
    #          | Main Window                   |
    #          |                               |
    #          |                               |
    #          |Toggle button's (x, y)         |
    #          |(relative to parent window)    |
    #          | |                             |
    #          | V                             |
    #          |  .........................    |
    # Popover  | |  Toggle Button          |   |
    # window's | |                         |   |
    # (x, y)---+> .........................    |
    #          |(window will be here) |
    #          |                               |
    #          |                               |
    #          ---------------------------------
    #   Popover Window's screen coordinates:
    #   x = Window's origin x + Toggle Button's relative x
    #   y = Window's origin y + Toggle Button's relative y + Toggle Button's
    #       height

    def _on_show(self, widget):
        rect = self._parent_button.get_allocation()
        main_window = self._parent_button.get_toplevel()
        [val, win_x, win_y] = main_window.get_window().get_origin()
        cal_x = win_x + rect.x
        cal_y = win_y + rect.y + rect.height

        [x, y] = self.apply_screen_coord_correction(cal_x, cal_y)
        self.move(x, y)

    # This function "tries" to correct calendar window position so that it is
    # not obscured when
    # a portion of main window is off-screen.
    # Known bug: If the main window is partially off-screen before Calendar
    # window
    # has been realized then get_allocation() will return rect of 1x1 in which
    # case
    # the calculations will fail & correction will not be applied
    def apply_screen_coord_correction(self, x, y):
        corrected_y = y
        corrected_x = x
        rect = self.get_allocation()
        screen_w = Gdk.Screen.width()
        screen_h = Gdk.Screen.height()

        delta_x = screen_w - (x + rect.width)
        delta_y = screen_h - (y + rect.height)
        if delta_x < 0:
            corrected_x += delta_x
            print("at x")
        if corrected_x < 0:
            corrected_x = 0

        button_rect = self._parent_button.get_allocation()
        window_width, window_height = \
            self._parent_button.get_toplevel().get_size()
        # print (y, button_rect.y, button_rect.height, )

        calc = (window_height - (button_rect.y + (button_rect.height * 2)))
        if delta_y < 0 or (calc < 0):
            btn_hgt = self._parent_button.get_allocation().height
            corrected_y = y - rect.height - btn_hgt
            print("at y")
        if corrected_y < 0:
            corrected_y = 0
        return [corrected_x, corrected_y]

    # "configure-event" callback of main window, try to move calendar window
    # along with main window.
    def on_window_config(self, widget, event):
        # Maybe better way to find the visiblilty
        if self.get_mapped():
            rect = self._parent_button.get_allocation()
            main_window = self._parent_button.get_toplevel()
            [val, win_x, win_y] = main_window.get_window().get_origin()
            cal_x = win_x + rect.x
            cal_y = win_y + rect.y + rect.height

            self.show_all()
            [x, y] = self.apply_screen_coord_correction(cal_x, cal_y)
            self.move(x, y)



# -*- Mode: python; coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-
#
# Copyright (C) 2015 - fossfreedom
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

from gi.repository import GObject
from gi.repository import Gio
from gi.repository import Gtk

from alttoolbar_preferences import CoverLocale


class AltControllerCategory(object):
    OTHER = 0
    LOCAL = 1
    ONLINE = 2
    PLAYLIST = 3


class AltControllerBase(GObject.Object):
    """
    base controller
    """

    def __init__(self, header):
        """
        Initialises the object.
        """
        self.header = header
        self.find = self.header.find  # convenience function

        self._pixbuf = None

        super(AltControllerBase, self).__init__()

    def get_category(self):
        """
           return the category type for the source
        """

        return AltControllerCategory.OTHER

    def get_gicon(self, source):
        """
          return the source icon
        :param source:
        :return:
        """

        if source.props.icon:
            return source.props.icon

        return None

    def valid_source(self, source):
        """
          returns bool if the given source is applicable to the controller
        """

        return False

    def update_controls(self, source):
        """
           update the button controls on the header
        """

        pass

    def remove_controls(self, container):
        """
          remove any controls that are contained in a container
        """
        for child in container.get_children():
            container.remove(child)

    def hide_controls(self, source):
        """
          hide controls for a given controller
        """

        pass

    def get_search_entry(self, toolbar_container):
        """
          find the GtkEntry field corresponding to the search entry
          returns 1. the GtkWidget containing the GtkEntry 
                  2. the GtkEntry
          returns None if nothing found
        """

        return None

    def get_toolbar(self, source):
        """
          return GtkWidget corresponding to the toolbar within the source
                 None if no toolbar
        """

        return None

    def moveto_searchbar(self, toolbar, widget, searchbar):
        """
          move from toolbar the widget and add to the searchbar
        """

        pass

    def set_library_labels(self):
        """
          set the centre library song-category button label
        """

        self.header.set_library_labels()


class AltGenericController(AltControllerBase):
    """
    generic controller for the headerbar (only)
    """
    __gtype_name = 'AltGenericController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltGenericController, self).__init__(header)

        self.centre_controls = {}
        self.end_controls = {}

    def get_category(self):
        return AltControllerCategory.LOCAL

    def hide_controls(self, source):
        val, view_button = self.header.has_button_with_label(source,
                                                             _('View All'))

        if val:
            view_button.set_visible(False)

    def get_toolbar(self, source):

        toolbar = self.find(source, 'RBSourceToolbar', 'by_name')
        print(toolbar)
        print(source)

        return toolbar

    def get_search_entry(self, container):
        if container is None:
            print("no container to search")
            return None, None
        search = self.find(container, 'RBSearchEntry', 'by_name')

        if not search:
            print("no RBSearchEntry found")
            return None, None

        entry = self.find(search, 'GtkEntry', 'by_name')
        print(entry)
        return search, entry

    def moveto_searchbar(self, toolbar, search, searchbar):
        toolbar.remove(search)
        toolbar.set_visible(False)

        searchbar.add(search)

    def update_controls(self, source):
        """
           update the button controls on the header
        """

        val, browser_button = self.header.is_browser_view(source)
        if not val:
            # if not a browser_view based source then default just to the title
            print("no browser view")
            self.header.set_library_box_sensitive(False)
        else:
            print("browser view found")
            browser_button.set_visible(False)
            self.header.set_library_box_sensitive(True)

        self.header.current_search_button = None

        toolbar = self.get_toolbar(source)
        if not toolbar:
            # there is no source-bar so the header is empty
            print("no toolbar so nothing left to do - cleanup endbox and exit")
            self.remove_controls(self.header.end_box)
            return

        self.hide_controls(toolbar)

        if source not in self.end_controls:
            # this is the first time for the source so extract the
            # RBSearchEntry
            print("first time around")
            controls = {}

            self.remove_controls(self.header.end_box)

            print(toolbar)  # should be the RBSourceToolbar
            search, entry = self.get_search_entry(toolbar)
            if not search:
                return

            if self.header.searchbar:
                self.header.searchbar.set_visible(False)

            # define a searchbar widget
            self.header.searchbar = Gtk.SearchBar.new()

            # we need to add this to the top of the source window
            # to-do this - find the first child and physically move this into
            # the second position in a box - the first position being the
            # searchbar
            children = source.get_children()
            print(children)
            # We assume the first container in a source is a GtkNotebook
            first = children[0]
            box = Gtk.Box()
            box.set_orientation(Gtk.Orientation.VERTICAL)
            box.pack_start(self.header.searchbar, False, True, 0)
            box.show_all()
            # so remove the notebook from the source
            Gtk.Container.remove(source, first)
            box.pack_start(first, True, True, 1)  # add the notebook to a box

            source.add(box)  # then add the box back to the source -
            # i.e. we added another parent

            self.header.register_moved_control(child=first,
                                               old_parent=source,
                                               new_parent=box)

            self.moveto_searchbar(toolbar, search, self.header.searchbar)
            self.header.searchbar.connect_entry(entry)
            # self.header.searchbar.show_all()
            self.header.searchbar.set_visible(False)

            search_button = Gtk.ToggleButton.new()
            sym = "preferences-system-search-symbolic"
            image = \
                Gtk.Image.new_from_icon_name(sym, Gtk.IconSize.SMALL_TOOLBAR)
            search_button.add(image)

            self.header.end_box.add(search_button)
            self.header.end_box.reorder_child(search_button, 0)
            search_button.show_all()
            search_button.connect('toggled', self.header.search_button_toggled)

            controls['searchbar'] = self.header.searchbar
            controls['search_button'] = search_button
            self.header.current_search_button = search_button
            self.end_controls[source] = controls
            print(controls)
        else:
            print("second time around")
            print(self.end_controls[source])
            search = self.end_controls[source]['searchbar']
            if self.header.searchbar:
                self.header.searchbar.set_visible(False)
            self.header.searchbar = search
            # self.header.searchbar.set_visible(True)

            self.remove_controls(self.header.end_box)
            search_button = self.end_controls[source]['search_button']
            self.header.current_search_button = search_button

            self.header.searchbar.show_all()
            self.header.searchbar.set_visible(search_button.get_active())

            self.header.end_box.add(search_button)
            self.header.end_box.reorder_child(search_button, 0)
            self.header.end_box.show_all()


class AltMusicLibraryController(AltGenericController):
    """
    music library controller
    """
    __gtype_name = 'AltMusicLibraryController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltMusicLibraryController, self).__init__(header)

    def valid_source(self, source):
        """
          override
        """

        return "LibrarySource" in type(source).__name__

    def hide_controls(self, source):
        super(AltMusicLibraryController, self).hide_controls(source)

        val, import_button = self.header.has_button_with_label(source,
                                                               _('Import'))

        if val:
            import_button.set_visible(False)


class AltSoundCloudController(AltGenericController):
    """
    sound-cloud controller
    """
    __gtype_name = 'AltSoundCloudController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltSoundCloudController, self).__init__(header)

        self._has_toolbar = None

    def valid_source(self, source):
        """
          override
        """

        return "SoundCloud" in type(source).__name__

    def get_category(self):
        return AltControllerCategory.ONLINE

    def get_toolbar(self, source):
        if self._has_toolbar:
            return self._has_toolbar

        search_box = self.find(source, 'box1', 'by_id')

        self._has_toolbar = search_box
        return search_box

    def moveto_searchbar(self, toolbar, widget, searchbar):
        """
          override - here we want to actually remove the toolbar from the
          source so get the parent
        """

        parent_grid = toolbar.get_parent()
        parent_grid.remove(toolbar)
        searchbar.add(toolbar)

        self.header.register_moved_control(child=toolbar,
                                           old_parent=parent_grid,
                                           new_parent=searchbar)


class AltCoverArtBrowserController(AltGenericController):
    """
    CoverArtBrowser controller
    """
    __gtype_name = 'AltCoverArtBrowserController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltCoverArtBrowserController, self).__init__(header)

        self._has_toolbar = None

    def valid_source(self, source):
        """
          override
        """

        return "CoverArtBrowser" in type(source).__name__

    def get_category(self):
        return AltControllerCategory.LOCAL

    def get_toolbar(self, source):
        if not self._has_toolbar:
            search_box = self.find(source, 'toolbar', 'by_id')
            self._has_toolbar = search_box

        return self._has_toolbar

    def moveto_searchbar(self, toolbar, widget, searchbar):
        """
          override - here we want to actually remove the toolbar from the
          source so get the parent
        """

        parent_grid = toolbar.get_parent()
        parent_grid.remove(toolbar)
        searchbar.add(toolbar)
        searchbar.show_all()

        self.header.register_moved_control(child=toolbar,
                                           old_parent=parent_grid,
                                           new_parent=searchbar)

    def get_search_entry(self, toolbar):
        """
          override - use the GtkEntry in the coverartbrowser
        """

        entrysearch = self.find(toolbar, 'entry_search_alignment', 'by_id')
        entry = self.find(entrysearch, 'GtkEntry', 'by_name')

        return entrysearch, entry


class AltCoverArtPlaySourceController(AltGenericController):
    """
    CoverArtPlaySource controller
    """
    __gtype_name = 'AltCoverArtPlaySourceController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltCoverArtPlaySourceController, self).__init__(header)

        self._has_toolbar = None

    def valid_source(self, source):
        """
          override
        """

        return "CoverArtPlaySource" in type(source).__name__

    def get_category(self):
        return AltControllerCategory.LOCAL

    def get_toolbar(self, source):
        if not self._has_toolbar:
            self._has_toolbar = self.find(source, 'RBButtonBar', 'by_name')

        print("############", self._has_toolbar)
        return self._has_toolbar


class AltQueueController(AltGenericController):
    """
    RB QueueSource controller
    """
    __gtype_name = 'AltQueueController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltQueueController, self).__init__(header)

        self._gicon = Gio.ThemedIcon(name='audio-x-queue-symbolic')

    def valid_source(self, source):
        return "RBPlayQueueSource" in type(source).__name__

    def get_gicon(self, source):
        return self._gicon


class AltErrorsController(AltGenericController):
    """
    RB ErrorsSource controller
    """
    __gtype_name = 'AltErrorsController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltErrorsController, self).__init__(header)

        self._gicon = Gio.ThemedIcon(name='dialog-error-symbolic')

        self._source_types = ["RBImportErrorsSource",
                              "RBMissingFilesSource"]

    def valid_source(self, source):
        print(type(source).__name__)
        for source_type in self._source_types:
            if source_type in type(source).__name__:
                return True

    def get_category(self):
        return AltControllerCategory.LOCAL

    def get_gicon(self, source):
        return self._gicon


class AltRadioController(AltGenericController):
    """
    RB RadioSource controller
    """
    __gtype_name = 'AltRadioController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltRadioController, self).__init__(header)

        self._gicon = Gio.ThemedIcon(name='audio-radio-symbolic')

    def valid_source(self, source):
        return "RBIRadioSource" in type(source).__name__

    def get_gicon(self, source):
        return self._gicon

    def get_category(self):
        return AltControllerCategory.ONLINE

    def set_library_labels(self):
        self.header.set_library_labels(song_label=_('Stations'))


class AltLastFMController(AltGenericController):
    """
    RB LastFMSource controller
    """
    __gtype_name = 'AltLastFMController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltLastFMController, self).__init__(header)

        self._libre_gicon = Gio.ThemedIcon(name='librefm-symbolic')
        self._lastfm_gicon = Gio.ThemedIcon(name='lastfm-symbolic')

    def valid_source(self, source):
        return "RBAudioscrobblerProfilePage" in type(source).__name__

    def get_gicon(self, source):
        # locale stuff
        cl = CoverLocale()
        cl.switch_locale(cl.Locale.RB)

        if source.props.name == _("Libre.fm"):
            return self._libre_gicon
        else:
            return self._lastfm_gicon

    def get_category(self):
        return AltControllerCategory.ONLINE


class AltPlaylistController(AltGenericController):
    """
    playlist controller
    """
    __gtype_name = 'AltPlaylistController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltPlaylistController, self).__init__(header)

        self._static_gicon = \
            Gio.ThemedIcon(name='audio-x-playlist-symbolic')
        self._auto_gicon = \
            Gio.ThemedIcon(name='audio-x-playlist-automatic-symbolic')

        self._toprated_gicon = Gio.ThemedIcon(name='starred-symbolic')
        self._recentlyadded_gicon = \
            Gio.ThemedIcon(name='audio-x-playlist-recently-added-symbolic')
        self._recentlyplayed_gicon = \
            Gio.ThemedIcon(name='audio-x-playlist-recently-played-symbolic')

    def valid_source(self, source):
        """
          override
        """
        return "PlaylistSource" in type(source).__name__

    def get_gicon(self, source):
        # locale stuff
        cl = CoverLocale()
        cl.switch_locale(cl.Locale.RB)
        print(source.props.name)
        if source.props.name == _('My Top Rated') \
                or source.props.name == 'My Top Rated':
            return self._toprated_gicon

        if source.props.name == _('Recently Added') \
                or source.props.name == 'Recently Added':
            return self._recentlyadded_gicon

        if source.props.name == _('Recently Played') \
                or source.props.name == 'Recently Played':
            return self._recentlyplayed_gicon

        if "StaticPlaylistSource" in type(source).__name__:
            return self._static_gicon
        else:
            return self._auto_gicon

    def get_category(self):
        return AltControllerCategory.PLAYLIST


class AltPodcastController(AltGenericController):
    """
    podcast controller
    """
    __gtype_name = 'AltPodcastController'

    def valid_source(self, source):
        """
          override
        """
        return 'RBPodcastMainSource' in type(source).__name__

    def get_category(self):
        return AltControllerCategory.LOCAL

    def set_library_labels(self):
        # locale stuff
        cl = CoverLocale()
        cl.switch_locale(cl.Locale.LOCALE_DOMAIN)

        self.header.set_library_labels(song_label=_('Podcasts'))


class AltStandardOnlineController(AltGenericController):
    """
      standard controller where we dont need specific customisation
    """
    __gtype_name = 'AltStandardOnlineController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltStandardOnlineController, self).__init__(header)

        self._source_types = ['MagnatuneSource',
                              'RBGriloSource',
                              'RadioBrowserSource']

    def valid_source(self, source):

        print(type(source).__name__)
        for source_type in self._source_types:
            if source_type in type(source).__name__:
                return True

        return False

    def get_category(self):
        return AltControllerCategory.ONLINE


class AltStandardLocalController(AltGenericController):
    """
      standard controller where we dont need specific customisation
    """
    __gtype_name = 'AltStandardLocalController'

    def __init__(self, header):
        """
        Initialises the object.
        """
        super(AltStandardLocalController, self).__init__(header)

        self._source_types = ['RBMtpSource']

    def valid_source(self, source):

        print(type(source).__name__)
        for source_type in self._source_types:
            if source_type in type(source).__name__:
                return True

        return False

    def get_category(self):
        return AltControllerCategory.LOCAL


class AltAndroidController(AltGenericController):
    '''
    android controller
    '''
    __gtype_name = 'AltAndroidController'

    def valid_source(self, source):
        '''
          override
        '''
        return 'RBAndroidSource' in type(source).__name__

    def get_category(self):
        return AltControllerCategory.LOCAL



# -*- Mode: python; coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-
#
# Copyright (C) 2015 - fossfreedom
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

# define plugin

import gettext
import locale
import os
import shutil
import sys

import rb
from gi.repository import GObject
from gi.repository import Gio
from gi.repository import Gtk
from gi.repository import PeasGtk
from gi.repository import RB


class GSetting:
    """
    This class manages the different settings that the plugin has to
    access to read or write.
    """
    # storage for the instance reference
    __instance = None

    class __impl:
        """ Implementation of the singleton interface """

        # below public variables and methods that can be called for GSetting
        def __init__(self):
            """
            Initializes the singleton interface, assigning all the constants
            used to access the plugin's settings.
            """
            self.Path = self._enum(
                PLUGIN='org.gnome.rhythmbox.plugins.alternative_toolbar')

            self.PluginKey = self._enum(
                DISPLAY_TYPE='display-type',
                START_HIDDEN='start-hidden',
                SHOW_COMPACT='show-compact',
                PLAYING_LABEL='playing-label',
                VOLUME_CONTROL='volume-control',
                INLINE_LABEL='inline-label',
                COMPACT_PROGRESSBAR='compact-progressbar',
                ENHANCED_SIDEBAR='enhanced-sidebar',
                EXPANDERS='expanders',
                SHOW_TOOLTIPS='show-tooltips',
                ENHANCED_PLUGINS='enhanced-plugins',
                REPEAT_TYPE='repeat-type',
                SOURCE_TOOLBAR='show-source-toolbar'
            )

            self.setting = {}

        def get_setting(self, path):
            """
            Return an instance of Gio.Settings pointing at the selected path.
            """
            try:
                setting = self.setting[path]
            except:
                self.setting[path] = Gio.Settings.new(path)
                setting = self.setting[path]

            return setting

        def get_value(self, path, key):
            """
            Return the value saved on key from the settings path.
            """
            return self.get_setting(path)[key]

        def set_value(self, path, key, value):
            """
            Set the passed value to key in the settings path.
            """
            self.get_setting(path)[key] = value

        def _enum(self, **enums):
            """
            Create an enumn.
            """
            return type('Enum', (), enums)

    def __init__(self):
        """ Create singleton instance """
        # Check whether we already have an instance
        if GSetting.__instance is None:
            # Create and remember instance
            GSetting.__instance = GSetting.__impl()

        # Store instance reference as the only member in the handle
        self.__dict__['_GSetting__instance'] = GSetting.__instance

    def __getattr__(self, attr):
        """ Delegate access to implementation """
        return getattr(self.__instance, attr)

    def __setattr__(self, attr, value):
        """ Delegate access to implementation """
        return setattr(self.__instance, attr, value)


class CoverLocale:
    """
    This class manages the locale
    """
    # storage for the instance reference
    __instance = None

    class __impl:
        """ Implementation of the singleton interface """

        # below public variables and methods that can be called for CoverLocale
        def __init__(self):
            """
            Initializes the singleton interface, assigning all the constants
            used to access the plugin's settings.
            """
            self.Locale = self._enum(
                RB='rhythmbox',
                LOCALE_DOMAIN='alternative-toolbar')

        def switch_locale(self, locale_type):
            """
            Change the locale
            """
            locale.setlocale(locale.LC_ALL, '')
            locale.bindtextdomain(locale_type, RB.locale_dir())
            locale.textdomain(locale_type)
            gettext.bindtextdomain(locale_type, RB.locale_dir())
            gettext.textdomain(locale_type)
            gettext.install(locale_type)

        def get_locale(self):
            """
            return the string representation of the users locale
            for example
            en_US
            """
            return locale.getdefaultlocale()[0]

        def _enum(self, **enums):
            """
            Create an enumn.
            """
            return type('Enum', (), enums)

    def __init__(self):
        """ Create singleton instance """
        # Check whether we already have an instance
        if CoverLocale.__instance is None:
            # Create and remember instance
            CoverLocale.__instance = CoverLocale.__impl()

        # Store instance reference as the only member in the handle
        self.__dict__['_CoverLocale__instance'] = CoverLocale.__instance

    def __getattr__(self, attr):
        """ Delegate access to implementation """
        return getattr(self.__instance, attr)

    def __setattr__(self, attr, value):
        """ Delegate access to implementation """
        return setattr(self.__instance, attr, value)


class Preferences(GObject.Object, PeasGtk.Configurable):
    """
    Preferences for the Plugins. It holds the settings for
    the plugin and also is the responsible of creating the preferences dialog.
    """
    __gtype_name__ = 'AlternativeToolbarPreferences'
    object = GObject.property(type=GObject.Object)

    def __init__(self):
        """
        Initialises the preferences, getting an instance of the settings saved
        by Gio.
        """
        GObject.Object.__init__(self)
        self.gs = GSetting()
        self.plugin_settings = self.gs.get_setting(self.gs.Path.PLUGIN)

    def do_create_configure_widget(self):
        """
        Creates the plugin's preferences dialog
        """
        print("DEBUG - create_display_contents")
        # create the ui
        self._first_run = True

        cl = CoverLocale()
        cl.switch_locale(cl.Locale.LOCALE_DOMAIN)
        builder = Gtk.Builder()
        builder.set_translation_domain(cl.Locale.LOCALE_DOMAIN)
        builder.add_from_file(rb.find_plugin_file(self,
                                                  'ui/altpreferences.ui'))
        builder.connect_signals(self)

        # bind the toggles to the settings
        start_hidden = builder.get_object('start_hidden_checkbox')

        start_hidden.set_active(
            not self.plugin_settings[self.gs.PluginKey.START_HIDDEN])
        start_hidden.connect('toggled', self._start_hidden_checkbox_toggled)

        self._show_compact = builder.get_object('show_compact_checkbox')
        self.plugin_settings.bind(self.gs.PluginKey.SHOW_COMPACT,
                                  self._show_compact, 'active',
                                  Gio.SettingsBindFlags.DEFAULT)

        self._show_compact.connect('toggled',
                                   self._show_compact_checkbox_toggled)

        self._playing_label = builder.get_object('playing_label_checkbox')
        self.plugin_settings.bind(self.gs.PluginKey.PLAYING_LABEL,
                                  self._playing_label, 'active',
                                  Gio.SettingsBindFlags.DEFAULT)

        self._inline_label = builder.get_object('inline_label_checkbox')
        self.plugin_settings.bind(self.gs.PluginKey.INLINE_LABEL,
                                  self._inline_label, 'active',
                                  Gio.SettingsBindFlags.DEFAULT)

        volume_control = builder.get_object('volume_control_checkbox')
        self.plugin_settings.bind(self.gs.PluginKey.VOLUME_CONTROL,
                                  volume_control, 'active',
                                  Gio.SettingsBindFlags.DEFAULT)

        self._compact_control = builder.get_object('compact_checkbox')
        self.plugin_settings.bind(self.gs.PluginKey.COMPACT_PROGRESSBAR,
                                  self._compact_control, 'active',
                                  Gio.SettingsBindFlags.DEFAULT)

        self._enhanced_sidebar = builder.get_object(
            'enhanced_sidebar_checkbox')
        self.plugin_settings.bind(self.gs.PluginKey.ENHANCED_SIDEBAR,
                                  self._enhanced_sidebar, 'active',
                                  Gio.SettingsBindFlags.DEFAULT)

        self._show_tooltips = builder.get_object('tooltips_checkbox')
        self.plugin_settings.bind(self.gs.PluginKey.SHOW_TOOLTIPS,
                                  self._show_tooltips, 'active',
                                  Gio.SettingsBindFlags.DEFAULT)

        self._enhanced_plugins = \
            builder.get_object('enhanced_plugins_checkbox')
        self.plugin_settings.bind(self.gs.PluginKey.ENHANCED_PLUGINS,
                                  self._enhanced_plugins, 'active',
                                  Gio.SettingsBindFlags.DEFAULT)

        modern_switch = builder.get_object('modern_switch')
        # modern_switch.connect('state-set', self._modern_switch_state)
        modern_switch.connect('notify', self._modern_switch_state)

        # Determine what type of toolbar is to be displayed
        default = Gtk.Settings.get_default()
        display_type = self.plugin_settings[self.gs.PluginKey.DISPLAY_TYPE]

        if display_type == 0:
            if (not default.props.gtk_shell_shows_app_menu) or \
                    default.props.gtk_shell_shows_menubar:
                modern_switch.set_active(False)
            else:
                modern_switch.set_active(True)
        elif display_type == 1:
            modern_switch.set_active(True)
        else:
            modern_switch.set_active(False)

        if modern_switch.get_active():
            self._show_compact.set_active(True)

        self._show_compact_checkbox_toggled(self._show_compact)

        infobar = builder.get_object('infobar')
        button = infobar.add_button(_("Restart"), 1)
        # restart_button = builder.get_object('restart_button')
        button.connect('clicked', self._restart_button_clicked)

        self._first_run = False

        return builder.get_object('preferences_box')

    def _restart_button_clicked(self, *args):
        exepath = shutil.which('rhythmbox')
        os.execl(exepath, exepath, *sys.argv)

    def _start_hidden_checkbox_toggled(self, toggle_button):
        self.plugin_settings[self.gs.PluginKey.START_HIDDEN] = \
            not toggle_button.get_active()

    def _show_compact_checkbox_toggled(self, toggle_button):
        enabled = toggle_button.get_active()

        self._show_tooltips.set_sensitive(enabled)
        self._inline_label.set_sensitive(enabled)
        self._playing_label.set_sensitive(enabled)
        self._compact_control.set_sensitive(enabled)

    def _modern_switch_state(self, switch, param):
        state = switch.get_active()
        self._show_compact.set_sensitive(not state)

        if state:
            self._show_compact.set_active(True)
            self.plugin_settings[self.gs.PluginKey.DISPLAY_TYPE] = 1
        else:
            self.plugin_settings[self.gs.PluginKey.DISPLAY_TYPE] = 2
